import sys

from collections import namedtuple
from os.path import abspath, dirname, join, isdir
from os import environ, makedirs, walk

from . import __version__, __application__, isWindows
from .arguments import profile

from .Lib.IzzyLib import logging

from PyQt5.QtCore import QStandardPaths
from PyQt5.QtWebEngineWidgets import QWebEngineProfile


if hasattr(sys, '_MEIPASS'):
	scriptpath = abspath(sys._MEIPASS)
else:
	scriptpath = abspath(dirname(__file__))

home = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
appdata = QStandardPaths.writableLocation(QStandardPaths.AppDataLocation)
temp = QStandardPaths.writableLocation(QStandardPaths.TempLocation)
default_config_dir = join(appdata, 'barkshark/qtweb')
datapath = abspath(environ.get('CONFIG', default_config_dir))

variables = {
	'profile': profile,
	'scriptpath': scriptpath,
	'datapath': datapath,
	'database': join(datapath, profile, 'database.sqlite3'),
	'profilepath': join(datapath, profile),
	'localweb': join(scriptpath, 'localweb'),
	'resources': join(scriptpath, 'resources'),
	'tangopath': join(scriptpath, '3rdparty', 'Tango', 'scalable'),
	'iconpath': join(scriptpath, 'resources', 'icons'),
	'downloadpath': abspath(join(home, 'Downloads')),
	'lock': join(datapath, profile, 'lock.pid'),
	'home': home,
	'temp': join(temp, 'qtweb'),
	'local': 'qtweb'
}


default_vars = {
	'homepage': 'https://ddg.gg',
	'nitter': 'https://nitter.net',
	'mhost': '127.0.0.1',
	'mport': 8008
}


# Set various variables
var = namedtuple('Variables', variables.keys())(*variables.values())
default = namedtuple('Default', default_vars.keys())(*default_vars.values())

## create necessary data dirs
if not isdir(var.profilepath):
	makedirs(var.profilepath, exist_ok=True)

if not isdir(var.temp):
	makedirs(var.temp, exist_ok=True)

if not isdir(join(var.datapath, 'UserScripts')):
	makedirs(join(var.datapath, 'UserScripts'), exist_ok=True)

environ['QTWEBENGINE_DICTIONARIES_PATH'] = join(var.resources, 'dictionaries')
