from collections import namedtuple
from dataclasses import make_dataclass, field, asdict
from datetime import datetime

from ..Lib.IzzyLib.misc import boolean
from ..Lib.IzzyLib import logging

from .connection import db, tables, ParseData
from ..functions import psl, DotDict

from PyQt5.QtGui import QPalette, QColor


# todo: add column to specify type instead of just sending everything through Boolean
def config(key=None, default=None):
	cache = db.cache['config']
	if len(cache.items()) == 0:
		rows = db.fetch('config', single=False)

		for row in rows:
			cache.store(row.key, row)

	if not key:
		rows = cache.values()
		return {row.key: boolean(row.value, fail=False) for row in rows}

	row = db.cache['config'].fetch(key)

	if not row:
		row = db.fetch('config', key=key)
		db.cache['config'].store(key, row)

	return boolean(row.value, fail=False) if row else default


def permissions(domain=None, defaults=False):
	domain = 'HopefullyThisShouldFixTheDamnIssue' if domain == '' else domain
	cache = db.cache['siteoptions']
	data = []

	if len(cache.items()) == 0:
		rows = db.fetch('siteoptions', single=False)
		for row in rows:
			cache.store(row.domain, row)

	if domain:
		rows = [db.fetch('siteoptions', domain=domain)]
		db.cache['siteoptions'].store(domain, rows[0])

	else:
		rows = [db.cache['siteoptions'].fetch(domain)] if domain else cache.values()

	if rows == [None]:
		rows = None

	if not rows and defaults:
		row = DotDict()

		for k, v in tables.siteoptions.items():
			default = v.get('default')
			row[k] = default if default else None

		return row

	else:
		for row in rows:
			for k, v in row.items():
				row[k] = boolean(v) if k not in ['id', 'domain', 'lastupdate'] else v

			if domain:
				return row

			data.append(row)

	return data


def mastoaccount(name=None, default=False):
	cache = db.cache['mastodon']
	if len(cache.items()) == 0:
		rows = db.fetch('mastodon', single=False)

		for row in rows:
			cache.store(row.fullname, row)

	if not name and not default:
		return cache.values()

	row = db.cache['mastodon'].fetch(name if not name and default else get.config('defaccount'))

	if not row:
		row = db.fetch('mastodon', fullname=name)
		db.cache['mastodon'].store(name, row)

	return row


def search(keyword):
	cache = db.cache['search']
	row = cache.fetch(keyword)
	default = config('search')

	if not row:
		row = db.fetch('search', keyword=keyword)

	if not row:
		row = db.fetch('search', keyword=default)
		keyword = default

	if not row:
		logging.error('Not a search keyword:', keyword)
		return

	cache.store(keyword, row)

	return row


def theme(name=None):
	if name:
		row = db.fetch('themes', name=name)

		if not row:
			logging.error('Cannot find color theme:', name)
			return

		row.palette = db.window.themes.ColorToPalette(row.asdict())

		return row

	else:
		rows = db.fetch('themes', single=False)

		for row in rows:
			row.palette = db.window.themes.ColorToPalette(row.asdict())

		return rows
