import sys

from os import remove
from os.path import isfile, join
from pathlib import Path
from shutil import copyfile
from datetime import datetime
from dataclasses import make_dataclass, field, asdict

from ..Lib.IzzyLib import logging
from ..Lib.IzzyLib.misc import boolean

from . import put
from .connection import db, DB, SearchEngines

from .. import dbversion
from ..config import var

from PyQt5.QtCore import QDateTime


def run(version):
	global db
	backupdb = var.database + f'.backup-{version}'
	version = int(version)

	if not isfile(backupdb):
		copyfile(var.database, backupdb)

	else:
		## Close database, restore backup, and re-open it
		db.close()
		remove(var.database)
		copyfile(backupdb, var.database)
		db = DB(var.database)

	if version < 202007090450:
		logging.info('Updating to 202007030754')
		db.query('ALTER TABLE siteoptions ADD COLUMN mastodon BOOLEAN DEFAULT 0')
		db.query('ALTER TABLE siteoptions ADD COLUMN lastupdate DATETIME')

		for row in db.fetch('siteoptions', single=False):
			if not row.lastupdate:
				db.update('siteoptions', row.id, {'lastupdate': datetime.now()})

		db.UpVersion(202007090450)

	if version < 202007110621:
		logging.info('Updating to 2020071110621')
		db.CreateTable('mastodon')
		db.UpVersion(202007110621)

	if version < 202007200656:
		db.CreateTable('search')

		for engine in SearchEngines:
			db.insert('search', engine)

		db.insert('config', {'key': 'search', 'value': 'ddg'})
		db.UpVersion(202007200656)

	if version < 202007290727:
		db.query('ALTER TABLE tabs ADD COLUMN tabid INT DEFAULT 0')
		db.UpVersion(202007290727)

	if version < 202008042122:
		db.AddColumn('siteoptions', 'allowhttp', 'boolean', default=1)

		for row in db.fetch('siteoptions', single=False):
			row.allowhttp = 1
			row.Update()

		db.UpVersion(202008042122)

	if version < 202008052224:
		db.AddColumn('siteoptions', 'capture', 'boolean', default=0)

		for row in db.fetch('siteoptions', single=False):
			row.capture = 0
			row.Update()

		db.UpVersion(202008052224)

	if version < 202008150250:
		for row in db.fetch('siteoptions', single=False):
			row.capture = 0
			row.Update()

		db.UpVersion(202008150250)


	logging.info(f'Updated database to version {dbversion} :3')
