from datetime import datetime
from ..Lib.IzzyLib import logging

from . import get
from .connection import db, tables


def bookmark(args):
	updatetime = datetime.now()
	new_data = {}
	fields = ['name', 'url', 'description', 'category']
	query = db.fetch('bookmarks', rowid = args['id']) if args.get('id') else None

	for key, value in args.items():
		if value in ['None', 'none', '']:
			value = None

		if key in fields:
			new_data[key] = value

	new_data['lastupdate'] = updatetime


	if query:
		return db.update('bookmarks', query.id, new_data)

	else:
		return db.insert('bookmarks', new_data)


def config(key, value):
	row = db.fetch('config', key=key)

	if db.cache['config'].fetch(key):
		db.cache['config'].remove(key)

	if not row:
		return db.insert('config', {'key': key, 'value': str(value)})
	else:
		return db.update('config', row.id, {'value': str(value)})


def permissions(domain, data):
	new_data = {'domain': domain, 'lastupdate': datetime.now()}
	fields = db.GetFields('siteoptions', ['lastupdate', 'id', 'domain'])
	row = db.fetch('siteoptions', domain=domain)

	for key, value in data.items():
		if value in ['None', 'none', '']:
			value = None

		if key in fields:
			new_data[key] = value

	if db.cache['siteoptions'].fetch(domain):
		db.cache['siteoptions'].remove(domain)

	if row:
		return True if row.Update(new_data) else False

	else:
		return True if db.insert('siteoptions', new_data) else False


def mastoaccount(data):
	updatetime = datetime.now()
	fields = ['username', 'displayname', 'domain', 'fullname', 'apikey', 'tootlimit', 'avatar']
	row = db.fetch('mastodon', domain=data.get('domain'))
	newdata = {}

	for k, v in data.items():
		if k in fields:
			newdata[k] = v

	newdata['lastupdate'] = updatetime

	if row:
		return db.update('mastodon', row.id, newdata)

	else:
		return db.insert('mastodon', newdata)


def search(data):
	fields = ['name', 'keyword', 'url']
	newdata = {}
	keyword = data.get('keyword')

	row = db.fetch('search', keyword=keyword)

	for k, v in data.items():
		if k in fields:
			newdata[k] = v

	if db.cache['search'].fetch(keyword):
		db.cache['search'].remove(keyword)

	if row:
		return db.update('search', row.id, newdata)

	else:
		return db.insert('search', newdata)


def tab(tabid, url, title):
	tabrow = db.fetch('tabs', url=url)

	if not tabrow:
		db.insert('tabs', {'title': title, 'url': url, 'tabid': tabid})

	else:
		tabrow.Update({'title': title, 'url': url, 'tabid': tabid})


def theme(name, palette):
	row = get.theme(name)
	data = db.window.themes.PaletteToColor(palette)

	if row:
		row.Update(data)

	else:
		db.insert('themes', data)
