from .Lib.IzzyLib import logging

from . import isWindows
from .database import get, put

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import QStyleFactory, QApplication


class ColorTheme(object):
	def __init__(self, window):
		self.app = window.app
		self.window = window
		self.default_style = self.app.style().objectName().lower()
		self.default_palette = self.app.style().standardPalette()
		self.appcolor = self.app.palette()
		self.colors = [
			'Window', 'WindowText', 'Base', 'AlternateBase', 'ToolTipBase', 'ToolTipText',
			'Button', 'ButtonText', 'Text', 'Highlight', 'HighlightedText', 'Link',
			'Light', 'Midlight', 'Dark', 'Mid', 'Shadow', 'LinkVisited', 'BrightText'
		]
		self.colors.sort()

		logging.verbose('Default style:', self.default_style)
		logging.verbose('Available styles:', ', '.join(QStyleFactory.keys()))


	def ChangeTheme(self, theme=None):
		theme = 'Default' if not theme else theme

		if theme == 'Default':
			palette = self.default_palette

		else:
			row = get.theme(theme)

			if not row:
				logging.error('Not a valid theme:', theme)
				return

			palette = row.palette

		logging.verbose('Changing color theme to', theme)
		put.config('theme', theme)
		self.app.setPalette(palette)

		if isWindows:
			if theme != 'Default':
				self.app.setStyle('fusion')

			else:
				self.app.setStyle(self.default_style)

			return


	def ColorToPalette(self, data):
		palette = self.app.style().standardPalette()

		for name, color in data.items():
			if name in self.colors and color != None:
				palette.setColor(getattr(QPalette, name), QColor(color))

		return palette


	def PaletteToColor(self, palette):
		data = {}

		for color in self.colors:
			data[color] = getattr(palette, color[0].lower() + ''.join(color[1:]))().color().name()

		return data
